@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -58,6 +56,7 @@
 
 extern dsBool				gResetSession;
 extern sInt32				gProcessPID;
+extern uInt32				gTranslateBit;
 
 //------------------------------------------------------------------------------------
 //	* CMessaging
@@ -169,7 +168,7 @@ sInt32 CMessaging::OpenCommPort ( void )
 #ifndef SERVERINTERNAL
 	siStatus	= eMemoryAllocError;
 
-	fCommPort = new CClientEndPoint( kDSServiceName );
+	fCommPort = new CClientEndPoint( kDSStdMachPortName );
 	if ( fCommPort != nil )
 	{
 		siStatus = fCommPort->Initialize();
@@ -341,13 +340,13 @@ sInt32 CMessaging::GetReplyMessage ( void )
 		
 		if (result == eServerSendError)
 		{
-			syslog(LOG_INFO,"DirectoryService Framework::CMessaging::Mach msg receiver error - out of order mach msgs.");
+			syslog(LOG_ALERT,"DirectoryService Framework::CMessaging::Mach msg receiver error - out of order mach msgs.");
 		}
 	
 		//if the mach reply failed due to an interrupt then we need to reset the entire session
 		if (result == eDSCannotAccessSession)
 		{
-			syslog(LOG_INFO,"DirectoryService Framework::CMessaging::Mach msg receiver interrupt error = %d", result);
+			syslog(LOG_ALERT,"DirectoryService Framework::CMessaging::Mach msg receiver interrupt error = %d", result);
 		
 			gResetSession = true;
 		}
@@ -410,10 +409,8 @@ sInt32 CMessaging::SendInlineMessage ( uInt32 inMsgType )
 	aMsgData->type.msgt_name		= inMsgType;
 	aMsgData->type.msgt_size		= 32;
 	aMsgData->type.msgt_number		= 0;
-	aMsgData->type.msgt_inline		= true;
-	aMsgData->type.msgt_longform	= false;
-	aMsgData->type.msgt_deallocate	= false;
-	aMsgData->type.msgt_unused		= 0;
+	aMsgData->type.msgt_translate	= 0;
+	aMsgData->fPID					= 0; // set the pid to 0 so we know it was internal dispatch
 
     handler.HandleRequest(&aMsgData);
 	
@@ -446,17 +443,14 @@ sInt32 CMessaging::SendInlineMessage ( uInt32 inMsgType )
 		fMsgData->type.msgt_name		= inMsgType;
 		fMsgData->type.msgt_size		= 32;
 		fMsgData->type.msgt_number		= 0;
-		fMsgData->type.msgt_inline		= true;
-		fMsgData->type.msgt_longform	= false;
-		fMsgData->type.msgt_deallocate	= false;
-		fMsgData->type.msgt_unused		= 0;
-	
+		fMsgData->type.msgt_translate	= gTranslateBit;
+		
 		result = this->SendServerMessage();
 	
 		//if the mach send was interrupted then we need to reset the entire session
 		if (result == eDSCannotAccessSession)
 		{
-			syslog(LOG_INFO,"DirectoryService Framework::CMessaging::Mach msg send interrupt error = %d.",result);
+			syslog(LOG_ALERT,"DirectoryService Framework::CMessaging::Mach msg send interrupt error = %d.",result);
 	
 			gResetSession = true;
 		}
@@ -1246,6 +1240,24 @@ void CMessaging::Lock ( void )
 #endif
 } // Lock
 
+//------------------------------------------------------------------------------------
+//	* ResetMessageBlock
+//------------------------------------------------------------------------------------
+
+void CMessaging::ResetMessageBlock( void )
+{
+	// let's free and reallocate the block if it isn't the default block size so we don't grow memory
+	if( kMsgBlockSize != fMsgData->fDataSize )
+	{
+		free( fMsgData );
+		fMsgData = (sComData *)::calloc( 1, sizeof( sComData ) + kMsgBlockSize );
+		if ( fMsgData != nil )
+		{
+			fMsgData->fDataSize		= kMsgBlockSize;
+			fMsgData->fDataLength	= 0;
+		}				
+	}
+} // ResetMessageBlock
 
 //------------------------------------------------------------------------------------
 //	* Unlock
@@ -1266,6 +1278,7 @@ void CMessaging::Unlock ( void )
 		{
 			if ( fLock != nil )
 			{
+				ResetMessageBlock();
 				fLock->Signal();
 			}
 		}
@@ -1274,6 +1287,7 @@ void CMessaging::Unlock ( void )
 	{
 		if ( fLock != nil )
 		{
+			ResetMessageBlock();
 			fLock->Signal();
 		}
 	}
@@ -1384,9 +1398,7 @@ bool CMessaging::IsThreadUsingInternalDispatchBuffering( OSType inThreadSig )
 {
 	bool	isInternalDispatchThread = false;
 	
-	if (	(inThreadSig == DSCThread::kTSHandlerThread) ||
-			(inThreadSig == DSCThread::kTSInternalHandlerThread) ||
-			(inThreadSig == DSCThread::kTSCheckpwHandlerThread) ||
+	if (	(inThreadSig == DSCThread::kTSMigHandlerThread) ||
 			(inThreadSig == DSCThread::kTSTCPConnectionThread) ||
 			(inThreadSig == DSCThread::kTSLauncherThread) ||
 			(inThreadSig == DSCThread::kTSPlugInHndlrThread) )
@@ -1396,4 +1408,4 @@ bool CMessaging::IsThreadUsingInternalDispatchBuffering( OSType inThreadSig )
 	
 	return(isInternalDispatchThread);
 } // IsThreadUsingInternalDispatchBuffering
-#endif
\ No newline at end of file
+#endif
