@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -30,7 +28,7 @@
 #ifndef __DSLogException_h__
 #define __DSLogException_h__	1
 
-#include "PrivateTypes.h"
+#include <DirectoryServiceCore/PrivateTypes.h>
 
 // Log macros
 
@@ -70,7 +68,7 @@
 	if ( aDSError != eDSNoErr )																													\
 	{																																			\
 		LOG4( kStdErr, "*** DS Error: File: %s. Line: %d. Error = %d. Thrown Error = %d\n", __FILE__, __LINE__, aDSError, inThrowThisError );	\
-		throw( inThrowThisError );																												\
+		throw( (sInt32)inThrowThisError );																												\
 	}																																			\
 } if (true)
 
