@@ -34,6 +34,7 @@
 #include <netinfo/ni.h>
 #include <netinfo/ni_util.h>
 #include <netinfo/ni_prot.h>
+#include <SystemConfiguration/SystemConfiguration.h>
 
 #include "CNetInfoPlugin.h"
 #include "DSUtils.h"
@@ -43,41 +44,30 @@
 #include "CString.h"
 #include "DSCThread.h"
 #include "DSEventSemaphore.h"
-#include "my_ni_pwdomain.h"
 #include "CSharedData.h"
 #include "CLog.h"
 #include "NiLib2.h"
+#include "DirServicesPriv.h"
+#include "netinfo_open.h"
 
 
 //PFIXextern	dsBool	gLocalNodeNotAvailable;
 
-/*
- * PRIVATE!
- * notifiation center SPI
- */
-#ifdef __cplusplus
-extern "C" {
-#endif
-extern uint32_t notify_get_state(int token, int *state);
-extern uint32_t notify_register_plain(const char *name, int *out_token);
-#ifdef __cplusplus
-}
-#endif
-
-#define NETINFO_BINDING_KEY "com.apple.system.netinfo.local.binding_change"
-
-#define BINDING_STATE_UNBOUND 0
-#define BINDING_STATE_BOUND 1
-#define BINDING_STATE_NETROOT 2
+extern	bool			gFirstNetworkUpAtBoot;
+extern	bool			gbBuildNILocalUserCache;
 
 // --------------------------------------------------------------------------------
 //	Globals
 
 DSEventSemaphore		*gKickNodeRequests	= nil;
 DSMutexSemaphore		*gNetInfoMutex		= nil;
+bool					gBlockNINetworkChange		= false;
+CFRunLoopTimerRef		CNetInfoPlugin::fTimerRef	= NULL;
+bool					CNetInfoPlugin::fHandlingNINetworkTransition = false;
+int						gCountToCheckLocalDataStamp	= 0;
 
-CNiNodeList	   *CNetInfoPlugin::fNiNodeList			= nil;
-FourCharCode	CNetInfoPlugin::fToken				= 0;
+CNiNodeList	   *gNiNodeList				= nil;
+FourCharCode	CNetInfoPlugin::fToken	= 0;
 
 static void DoNIPINetworkChange(CFRunLoopTimerRef timer, void *info);
 void DoNIPINetworkChange(CFRunLoopTimerRef timer, void *info)
@@ -95,20 +85,29 @@ CFStringRef NetworkChangeNIPICopyStringCallback( const void *item )
 	return CFSTR("NetworkChangeinNetInfoModule");
 }
 
+void SearchPolicyFoundNIParentNotification(SCDynamicStoreRef aSCDStore, CFArrayRef changedKeys, void *callback_argument)
+{                       
+	//need to re-check the notion of /NetInfo/root domain
+	if (gNiNodeList != nil)
+	{
+		gNiNodeList->AdjustForParent();
+	}
+}// DHCPChangeNotification
+
 // --------------------------------------------------------------------------------
 //	* CNetInfoPlugin ()
 // --------------------------------------------------------------------------------
 
 CNetInfoPlugin::CNetInfoPlugin ( FourCharCode inSig, const char *inName ) : CServerPlugin(inSig, inName)
 {
-	fNiNodeList				= nil;
 	fState					= kUnknownState;
 	fRegisterNodePtr		= nil;
 	fServerRunLoop			= nil;  //could be obtained directly now since a module not plugin
-	fTransitionCheckTime	= 0;
+	bFirstNetworkTransition	= true;
 
 	//need the signature for some static functions
 	CNetInfoPlugin::fToken = inSig;
+	fHandlingNINetworkTransition = false;
 	
 	if ( gKickNodeRequests == nil )
 	{
@@ -171,16 +170,14 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 			DSSemaphore			aCheckNIMutex;
                         
 			DBGLOG( kLogPlugin, "CNetInfoPlugin::Initialize - kick start netinfod initiated." );
-			//kick start netinfod here to ensure netinfod starts early upon boot/reboot?
-			//status = (int)do_open( nil, "local", &domain, true, 1, NULL, NULL );
-			status = (int)do_open( nil, "127.0.0.1/local", &domain, true, 1, NULL, NULL );
+			status = netinfo_open( NULL, ".", &domain, 10 );
 			if (status == NI_OK)
 			{
-				DBGLOG( kLogPlugin, "CNetInfoPlugin::Initialize - do_open on tag succeeded." );
+				DBGLOG( kLogPlugin, "CNetInfoPlugin::Initialize - netinfo_open on <.> succeeded." );
 			}
 			else
 			{
-				DBGLOG( kLogPlugin, "CNetInfoPlugin::Initialize - do_open on tag failed." );
+				DBGLOG( kLogPlugin, "CNetInfoPlugin::Initialize - netinfo_open on <.> failed." );
 			}
 			
 			DBGLOG( kLogPlugin, "CNetInfoPlugin::Initialize - netinfo notification check attempted." );
@@ -211,7 +208,7 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 			if (status != NOTIFY_STATUS_OK)
 			{
 				DBGLOG1( kLogPlugin, "CNetInfoPlugin::Initialize - notify_register_plain failed for netinfo binding (status %d).", status );
-				syslog(LOG_INFO,"CNetInfoPlugin::Initialize - notify_register_plain failed for netinfo binding (status %d)\n", status);
+				syslog(LOG_ALERT,"CNetInfoPlugin::Initialize - notify_register_plain failed for netinfo binding (status %d)\n", status);
 			}
 			else
 			{
@@ -224,7 +221,7 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 					if (status != NOTIFY_STATUS_OK)
 					{
 						DBGLOG1( kLogPlugin, "CNetInfoPlugin::Initialize - notify_get_state failed for netinfo binding (status %d).", status );
-						syslog(LOG_INFO,"CNetInfoPlugin::Initialize - notify_get_state failed for netinfo binding (status %d)\n", status);
+						syslog(LOG_ALERT,"CNetInfoPlugin::Initialize - notify_get_state failed for netinfo binding (status %d)\n", status);
 						status = -1;
 						break;
 					}
@@ -241,7 +238,7 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 					{
 						status = -1;
 						DBGLOG1( kLogPlugin, "CNetInfoPlugin::Initialize - notify_get_state failed over last minute (status %d).", status );
-						syslog(LOG_INFO,"CNetInfoPlugin::Initialize - notify_get_state failed over last minute (status %d)\n", status);
+						syslog(LOG_ALERT,"CNetInfoPlugin::Initialize - notify_get_state failed over last minute (status %d)\n", status);
 						break;
 					}
 			
@@ -255,7 +252,7 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 						if (status != NOTIFY_STATUS_OK)
 						{
 							DBGLOG2( kLogPlugin, "CNetInfoPlugin::Initialize - retry %d on notify_register_plain failed for netinfo binding (status %d)\n", numRetries, status );
-							syslog(LOG_INFO,"CNetInfoPlugin::Initialize - retry %d on notify_register_plain failed for netinfo binding (status %d)\n", numRetries, status);
+							syslog(LOG_ALERT,"CNetInfoPlugin::Initialize - retry %d on notify_register_plain failed for netinfo binding (status %d)\n", numRetries, status);
 							status = -1;
 							break;
 						}
@@ -278,11 +275,6 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 						break;
 				}
 			}
-			if (domain != nil)
-			{
-				ni_free(domain);
-				domain = nil;
-			}
 		}
 
 		if (siResult != eDSNoErr)
@@ -290,12 +282,12 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 			throw(siResult);
 		}
 
-		if ( fNiNodeList == nil )
+		if ( gNiNodeList == nil )
 		{
-			fNiNodeList = new CNiNodeList();
+			gNiNodeList = new CNiNodeList();
 		}
 
-		if ( fNiNodeList == nil ) throw( (sInt32)eMemoryAllocError );
+		if ( gNiNodeList == nil ) throw( (sInt32)eMemoryAllocError );
 
 		//Register the default local node immediately
 		pDataList = ::dsBuildFromPathPriv( kstrDefaultLocalNodeName, "/" );
@@ -303,7 +295,7 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 		if ( pDataList != nil )
 		{
 			//AddNode consumes the pDataList
-			fNiNodeList->AddNode( ".", pDataList, false ); //will be registered but not treated the same as others
+			gNiNodeList->AddNode( ".", pDataList, true, 1 ); //will be registered but not treated the same as others
 
 			pDataList = nil;
 		}
@@ -321,17 +313,20 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 			
 		}
 
-		fRegisterMutex.Wait();
-		if (fRegisterNodePtr == nil)
+		if (gFirstNetworkUpAtBoot) //no attempt to get netinfo hierarchy until AFTER first network transition
 		{
-			fRegisterNodePtr = new CNodeRegister( fPlugInSignature, fNiNodeList, false, this );
-			if ( fRegisterNodePtr != nil )
+			fRegisterMutex.Wait();
+			if (fRegisterNodePtr == nil)
 			{
-				//this call could throw
-				fRegisterNodePtr->StartThread();
+				fRegisterNodePtr = new CNodeRegister( fPlugInSignature, gNiNodeList, false, this );
+				if ( fRegisterNodePtr != nil )
+				{
+					//this call could throw
+					fRegisterNodePtr->StartThread();
+				}
 			}
+			fRegisterMutex.Signal();
 		}
-		fRegisterMutex.Signal();
 
 		// Turn off the failed flag and turn on active and inited flag
 		fState &= ~kFailedToInit;
@@ -362,6 +357,7 @@ sInt32 CNetInfoPlugin::Initialize ( void )
 
 sInt32 CNetInfoPlugin::PeriodicTask ( void )
 {
+	gCountToCheckLocalDataStamp++; //counts number of 30 second periodic tasks
 	return( eDSNoErr );
 } // PeriodicTask
 
@@ -455,6 +451,54 @@ sInt32 CNetInfoPlugin::ProcessRequest ( void *inData )
 			if ( (((sHeader *)inData)->fContextData) != nil )
 			{
 				fServerRunLoop = (CFRunLoopRef)(((sHeader *)inData)->fContextData);
+				
+				// now we have a server runloop, let's register our notification for Search Changes
+				CFStringRef service = CFStringCreateWithCString( NULL, "DirectoryService", kCFStringEncodingUTF8 );
+				
+				if( service )
+				{
+					SCDynamicStoreRef store = SCDynamicStoreCreate( NULL, service, SearchPolicyFoundNIParentNotification, NULL );
+					CFMutableArrayRef notifyKeys = CFArrayCreateMutable( kCFAllocatorDefault, 0, &kCFTypeArrayCallBacks );
+					
+					if( notifyKeys )
+					{
+						CFStringRef notify = CFStringCreateWithCString( NULL, kDSStdNotifySearchPolicyFoundNIParent, kCFStringEncodingUTF8 );
+						
+						if( notify )
+						{
+							CFArrayAppendValue( notifyKeys, notify );
+							CFRelease( notify );
+						}
+						else
+						{
+							CFRelease( notifyKeys );
+							notifyKeys = NULL;
+						}
+					}
+					
+					// let's add ourselves
+					if (store != NULL && notifyKeys != NULL)
+					{
+						SCDynamicStoreSetNotificationKeys(store, notifyKeys, NULL);
+						CFRunLoopSourceRef rls = SCDynamicStoreCreateRunLoopSource(NULL, store, 0);
+						if (rls != NULL)
+						{
+							CFRunLoopAddSource(fServerRunLoop, rls, kCFRunLoopDefaultMode);
+							CFRelease(rls);
+						}
+						else
+						{
+							syslog(LOG_ALERT,"Unable to add source to RunLoop for SystemConfiguration registration for DS Search Policy Changes");
+						}
+					}
+					
+					if( notifyKeys )
+						CFRelease(notifyKeys);
+					if( store )
+						CFRelease(store);
+
+					CFRelease(service);
+				}
 				return (siResult);
 			}
 		}
@@ -478,7 +522,18 @@ sInt32 CNetInfoPlugin::ProcessRequest ( void *inData )
 
 		if ( ((sHeader *)inData)->fType == kHandleNetworkTransition )
 		{
-			HandleMultipleNetworkTransitions();
+			if (!gBlockNINetworkChange)
+			{
+				HandleMultipleNetworkTransitions();
+			}
+		}
+		else if ( ((sHeader *)inData)->fType == kHandleSystemWillSleep )
+		{
+			SystemGoingToSleep();
+		}
+		else if ( ((sHeader *)inData)->fType == kHandleSystemWillPowerOn )
+		{
+			SystemWillPowerOn();
 		}
 		else
 		{
@@ -514,23 +569,47 @@ void CNetInfoPlugin::HandleMultipleNetworkTransitions ( void )
 	//we also don't want to re-init multiple times during this wait period
 	//however we do go ahead and fire off timers each time
 	//each call in here we update the delay time by 5 seconds
-	fTransitionCheckTime = time(nil) + 5;
-
 	if (fServerRunLoop != nil)
 	{
+		// invalidate any existing timer and free it, as we will add it again below
+		// this routine is called on main RunLoop so there is no need to mutex protect this variable
+		if( fTimerRef != NULL )
+		{
+			DBGLOG1( kLogPlugin, "T[%X] CNetInfoPlugin::HandleMultipleNetworkTransitions invalidating previous timer", pthread_self() );
+
+			CFRunLoopTimerInvalidate( fTimerRef );
+			CFRelease( fTimerRef );
+			fTimerRef = NULL;
+		}
+		
 		ptInfo = (void *)this;
 		CFRunLoopTimerContext c = {0, (void*)ptInfo, NULL, NULL, NetworkChangeNIPICopyStringCallback};
 	
-		CFRunLoopTimerRef timer = CFRunLoopTimerCreate(	NULL,
-														CFAbsoluteTimeGetCurrent() + 5,
-														0,
-														0,
-														0,
-														DoNIPINetworkChange,
-														(CFRunLoopTimerContext*)&c);
+		if (bFirstNetworkTransition)
+		{
+			fTimerRef = CFRunLoopTimerCreate(	NULL,
+											CFAbsoluteTimeGetCurrent(),
+											0,
+											0,
+											0,
+											DoNIPINetworkChange,
+											(CFRunLoopTimerContext*)&c);
+		}
+		else
+		{
+			DBGLOG1( kLogPlugin, "T[%X] CNetInfoPlugin::HandleMultipleNetworkTransitions, setting fHandlingNINetworkTransition to true", pthread_self() );
+			fHandlingNINetworkTransition = true;
+
+			fTimerRef = CFRunLoopTimerCreate(	NULL,
+											CFAbsoluteTimeGetCurrent() + 5,
+											0,
+											0,
+											0,
+											DoNIPINetworkChange,
+											(CFRunLoopTimerContext*)&c);
+		}
 	
-		CFRunLoopAddTimer(fServerRunLoop, timer, kCFRunLoopDefaultMode);
-		if (timer) CFRelease(timer);
+		CFRunLoopAddTimer(fServerRunLoop, fTimerRef, kCFRunLoopDefaultMode);
 	}
 } // HandleMultipleNetworkTransitions
 
@@ -542,23 +621,21 @@ void CNetInfoPlugin::ReDiscoverNetwork(void)
 {
 	sInt32	siResult	= eDSNoErr;
 	
-	//do something if the wait period has passed
-	if (time(nil) >= fTransitionCheckTime)
+	//do something if the wait period has passed or if first time
+	if ( ( fHandlingNINetworkTransition || bFirstNetworkTransition ) && !gBlockNINetworkChange )
 	{
-		if ( fNiNodeList != nil )
+		bFirstNetworkTransition = false;
+		if ( gNiNodeList != nil )
 		{
 			fRegisterMutex.Wait();
 			
 			if (fRegisterNodePtr == nil)
 			{
-				fRegisterNodePtr = new CNodeRegister( fPlugInSignature, fNiNodeList, true, this );
+				fRegisterNodePtr = new CNodeRegister( fPlugInSignature, gNiNodeList, true, this );
 				if ( fRegisterNodePtr != nil )
 				{
 					try
 					{
-						//set all the node flags to dirty
-						fNiNodeList->SetAllDirty();
-			
 						//this call could throw
 						fRegisterNodePtr->StartThread();
 					}
@@ -571,66 +648,97 @@ void CNetInfoPlugin::ReDiscoverNetwork(void)
 			}
 			else
 			{
-				//set all the node flags to dirty
-				//fNiNodeList->SetAllDirty(); // this is taken care of inside CNodeRegister
 				fRegisterNodePtr->Restart();
 			}
 			fRegisterMutex.Signal();
 		}
+		if (fHandlingNINetworkTransition)
+		{
+			fHandlingNINetworkTransition = false;
+		}
+		gbBuildNILocalUserCache = true;
 	}
 } // ReDiscoverNetwork
 
 
+//------------------------------------------------------------------------------------
+//	SystemGoingToSleep
+//------------------------------------------------------------------------------------
+
+void CNetInfoPlugin::SystemGoingToSleep( void )
+{
+	//set a network change blocking flag at sleep
+	gBlockNINetworkChange = true;
+}//SystemGoingToSleep
+
+//------------------------------------------------------------------------------------
+//	SystemWillPowerOn
+//------------------------------------------------------------------------------------
+
+void CNetInfoPlugin::SystemWillPowerOn( void )
+{
+	//reset a network change blocking flag at wake
+	gBlockNINetworkChange = false;
+}//SystemWillPowerOn
+
 // ---------------------------------------------------------------------------
 //	* SafeOpen
 //
 // ---------------------------------------------------------------------------
 
-sInt32 CNetInfoPlugin::SafeOpen ( const char   *inName,
-							 sInt32			inTimeoutSecs,
-							 ni_id		   *outNiDirID,
-							 void		  **outDomain,
-							 char		  **outDomainName )
+sInt32 CNetInfoPlugin::SafeOpen ( const char *inName, sInt32 inTimeoutSecs, char **outDomainName )
 {
 	sInt32		siResult		= eDSNoErr;
 	ni_status	niStatus		= NI_OK;
 	void	   *domain			= nil;
 	char	   *domName			= nil;
-	ni_id		domainID;
 	uInt32		localOrParent	= 0;
 	time_t		delayedNI		= 0;
 	tDataList  *pDataList		= nil;
 
+	gNiNodeList->Lock();
+
 	try
 	{
-		if ( fNiNodeList->IsOpen( inName, outDomain, outDomainName, outNiDirID ) == false )
+		if ( gNiNodeList->IsOpen( inName, outDomainName ) == false )
 		{
-			localOrParent = fNiNodeList->CheckForLocalOrParent( inName );
-			gNetInfoMutex->Wait();
+			localOrParent = gNiNodeList->CheckForLocalOrParent( inName );
 
-			delayedNI = time(nil) + 2; //normally ni_open will complete in under 2 secs
+			delayedNI = time(nil) + 2; //normally netinfo_open will complete in under 2 secs
 			if (localOrParent == 1)
 			{
-				niStatus = ::ni_open( nil, ".", &domain );
+				niStatus = netinfo_open( nil, ".", &domain, inTimeoutSecs );
+				if ( niStatus == NI_OK )
+				{
+					DBGLOG( kLogPlugin, "CNiPlugIn::SafeOpen: <netinfo_open> opening <.>" );
+				}
 				if ( delayedNI < time(nil) )
 				{
-					syslog(LOG_INFO,"SafeOpen::Call to ni_open was with fixed domain name: . and lasted %d seconds.", (uInt32)(2 + time(nil) - delayedNI));
+					syslog(LOG_ALERT,"SafeOpen::Call to netinfo_open was with fixed domain name: <.> and lasted %d seconds while timeout was %d", (uInt32)(2 + time(nil) - delayedNI), inTimeoutSecs);
 				}
 			}
 			else if (localOrParent == 2)
 			{
-				niStatus = ::ni_open( nil, "..", &domain );
+				niStatus = netinfo_open( nil, "..", &domain, inTimeoutSecs ); //netinfo_open will open relative to "." domain
+				if ( niStatus == NI_OK )
+				{
+					DBGLOG( kLogPlugin, "CNiPlugIn::SafeOpen: <netinfo_open> opening <..>" );
+				}
 				if ( delayedNI < time(nil) )
 				{
-					syslog(LOG_INFO,"SafeOpen::Call to ni_open was with fixed domain name: .. and lasted %d seconds.", (uInt32)(2 + time(nil) - delayedNI));
+					syslog(LOG_ALERT,"SafeOpen::Call to netinfo_open was with fixed domain name: <..> and lasted %d seconds while timeout was %d", (uInt32)(2 + time(nil) - delayedNI), inTimeoutSecs);
 				}
 			}
 			else
 			{
-				niStatus = ::ni_open( nil, inName, &domain );
+				niStatus = netinfo_open( nil, (char *)inName, &domain, inTimeoutSecs );
+				if ( niStatus == NI_OK )
+				{
+					DBGLOG1( kLogPlugin, "CNiPlugIn::SafeOpen: <netinfo_open> opening <%s>", inName );
+				}
 				if ( delayedNI < time(nil) )
 				{
-					syslog(LOG_INFO,"SafeOpen::Call to ni_open was with argument domain name: %s and lasted %d seconds.", inName, (uInt32)(2 + time(nil) - delayedNI));
+					syslog(LOG_ALERT,"SafeOpen::Call to netinfo_open was with argument domain name: %s and lasted %d seconds while timeout was %d", inName, (uInt32)(2 + time(nil) - delayedNI), inTimeoutSecs);
 				}
 			}
 			if ( niStatus != NI_OK )
@@ -638,29 +746,10 @@ sInt32 CNetInfoPlugin::SafeOpen ( const char   *inName,
 				throw( (sInt32)eDSOpenNodeFailed );
 			}
 
-			// Ping the domain to see if it's really alive
-			::ni_setreadtimeout( domain, inTimeoutSecs );
-			::ni_setabort( domain, 1 );
-
-			NI_INIT( &domainID );
-			domainID.nii_object = 0;
-			niStatus = ::ni_self( domain, &domainID );
-			if ( niStatus != NI_OK )
-			{
-				//check for nil and free if not
-				if (domain != nil)
-				{
-					::ni_free( domain );
-				}
-				throw( (sInt32)eDSOpenNodeFailed );
-			}
-
-			gNetInfoMutex->Signal();
-
 			if ( ::strcmp( inName, "/" ) == 0 )
 			{
-				domName = (char *)::calloc( ::strlen( "/NetInfo/root" ) + 1, sizeof( char ) );
-				::strcpy( domName, "/NetInfo/root" );
+				domName = (char *)::calloc( ::strlen( kstrRootNodeName ) + 1, sizeof( char ) );
+				::strcpy( domName, kstrRootNodeName );
 			}
 			else if ( ::strcmp( inName, "." ) == 0 )
 			{
@@ -670,26 +759,24 @@ sInt32 CNetInfoPlugin::SafeOpen ( const char   *inName,
 			{
 				char	* niDomName	= nil;
 				
-				gNetInfoMutex->Wait();
-				delayedNI = time(nil) + 2; //normally my_ni_pwdomain will complete in under 2 secs
-				niStatus = ::my_ni_pwdomain( domain, &niDomName );
+				delayedNI = time(nil) + 2; //normally netinfo_domainname will complete in under 2 secs
+				niDomName = netinfo_domainname( domain );
 				if ( delayedNI < time(nil) )
 				{
-					syslog(LOG_INFO,"SafeOpen::Call to my_ni_pwdomain was with argument domain name: %s and lasted %d seconds.", inName, (uInt32)(2 + time(nil) - delayedNI));
+					syslog(LOG_ALERT,"SafeOpen::Call to netinfo_domainname was with argument domain name: %s and lasted %d seconds.", inName, (uInt32)(2 + time(nil) - delayedNI));
 				}
-				gNetInfoMutex->Signal();
 				
-				if ( niStatus == NI_OK )
+				if ( niDomName != nil )
 				{
 					if ( ::strcmp( niDomName, "/" ) == 0 )
 					{
-						domName = (char *)::calloc( ::strlen( "/NetInfo/root" ) + 1, sizeof( char ) );
-						::strcpy( domName, "/NetInfo/root" );
+						domName = (char *)::calloc( ::strlen( kstrRootNodeName ) + 1, sizeof( char ) );
+						::strcpy( domName, kstrRootNodeName );
 					}
 					else
 					{
-						domName = (char *)::calloc( ::strlen( niDomName ) + ::strlen( "/NetInfo/root" ) + 1, sizeof( char ) );
-						::strcpy( domName, "/NetInfo/root" );
+						domName = (char *)::calloc( ::strlen( niDomName ) + ::strlen( kstrRootNodeName ) + 1, sizeof( char ) );
+						::strcpy( domName, kstrRootNodeName );
 						::strcat( domName, niDomName );
 					}
 					if (niDomName != nil)
@@ -706,74 +793,76 @@ sInt32 CNetInfoPlugin::SafeOpen ( const char   *inName,
 			}
 			else
 			{
-				domName = (char *)::calloc( ::strlen( "/NetInfo/root" ) + ::strlen( inName ) + 1, sizeof( char ) );
+				domName = (char *)::calloc( ::strlen( kstrRootNodeName ) + ::strlen( inName ) + 1, sizeof( char ) );
 
-				::strcpy( domName, "/NetInfo/root" );
+				::strcpy( domName, kstrRootNodeName );
 				::strcat( domName, inName );
 			}
 
-			fNiNodeList->Lock();
-			if ( fNiNodeList->IsOpen( inName, outDomain, outDomainName, outNiDirID ) == false )
+			//we are forcing the SetDomainInfo code to always create a node via AddNode
+			gNiNodeList->SetDomainInfo( inName, domain, domName );
+			
+			//we also register the node here since it was not already found in our node list
+			pDataList = ::dsBuildFromPathPriv( domName, "/" );
+			if ( pDataList != nil )
 			{
-				//we are forcing the SetDomainInfo code to always create a node via AddNode
-				//this gets around the refCount problem
-				fNiNodeList->SetDomainInfo( inName, domain, domName, &domainID );
-				fNiNodeList->UnLock();
-				
-				//we also register the node here since it was not already found in our node list
-				pDataList = ::dsBuildFromPathPriv( domName, "/" );
-				if ( pDataList != nil )
+				if (CServerPlugin::_RegisterNode( CNetInfoPlugin::fToken, pDataList, kDirNodeType ) == eDSNoErr)
 				{
-					if (CServerPlugin::_RegisterNode( CNetInfoPlugin::fToken, pDataList, kDirNodeType ) == eDSNoErr)
-					{
-						DBGLOG1( kLogPlugin, "CNetInfoPlugin::SafeOpen: Registering node that was not already registered %s.", domName );
-					}
-					::dsDataListDeallocatePriv( pDataList );
-					free( pDataList );
-					pDataList = nil;
+					DBGLOG1( kLogPlugin, "CNetInfoPlugin::SafeOpen: Registering node that was not already registered %s.", domName );
 				}
-
-				*outDomain		= domain;
-				*outDomainName	= strdup(domName);
-				::memcpy( outNiDirID, &domainID, sizeof( ni_id ) );
+				::dsDataListDeallocatePriv( pDataList );
+				free( pDataList );
+				pDataList = nil;
 			}
-			else
+
+			*outDomainName	= strdup(domName);
+
+			if (domName != nil)
 			{
-				fNiNodeList->UnLock();
-				if (domain != nil)
-				{
-					gNetInfoMutex->Wait();
-					ni_free(domain);
-					gNetInfoMutex->Signal();
-					domain = nil;
-				}
-				if (domName != nil)
-				{
-					free(domName);
-					domName = nil;
-				}
-				
+				free(domName);
+				domName = nil;
 			}
 		}
 	}
 
 	catch( sInt32 err )
 	{
-		gNetInfoMutex->Signal();
 		siResult = err;
 	}
 
 	catch ( ... )
 	{
-		gNetInfoMutex->Signal();
-		siResult = -12020;
+		siResult = eDSInvalidSession;
 	}
 
+	gNiNodeList->UnLock();
+
 	return( siResult );
 
 } // SafeOpen
 
 
+// ---------------------------------------------------------------------------
+//	* GetNIDomain
+//
+// ---------------------------------------------------------------------------
+
+void* CNetInfoPlugin::GetNIDomain(const char *inDomainName)
+{
+	void *outDomain = NULL;
+	
+	if (gNiNodeList != nil)
+	{
+		outDomain = gNiNodeList->RetrieveNode(inDomainName);
+	}
+	else
+	{
+		DBGLOG( kLogPlugin, "CNetInfoPlugin::GetNIDomain - NI node list is empty." );
+	}
+	
+	return outDomain;
+} // GetNIDomain
+
 // ---------------------------------------------------------------------------
 //	* SafeClose
 //
@@ -786,12 +875,14 @@ sInt32 CNetInfoPlugin::SafeClose ( const char   *inName )
 
 	gNetInfoMutex->Wait();
 	
-	domain = fNiNodeList->DeleteNode( inName );
-	if (domain != nil)
-	{
-		ni_free(domain);
-		domain = nil;
-	}
+	domain = gNiNodeList->DeleteNode( inName );
+	//if (domain != nil)
+	//{
+		//management of domains left to netinfo_open package now
+		//DBGLOG( kLogPlugin, "CNetInfoPlugin::SafeClose - ni_free on the ni domain after deleting node from list" );
+		//ni_free(domain);
+		//domain = nil;
+	//}
 	
 	gNetInfoMutex->Signal();
 	
